/* Copyright (c) 2009 Ondrej Donek.
   See the file LICENSE.txt for licensing information. */

(function() {
  
  /**
   * Prototype object for single note's category.
   * 
   * @param aArgs {mixed}
   * @type object
   */
  this.CategoryPrototype = function(aArgs)
  {
    if((aArgs instanceof CI.mozIStorageStatementRow)) {
      this._id = aArgs.id;
      this._parent_category_id = aArgs.parent_id;
      this._name = aArgs.name;
      this._description = aArgs.description;
      this._color = aArgs.color;
    }
  };
  this.CategoryPrototype.prototype = 
  {
    // id
    get mId() {
      return (typeof(this._id) != "undefined") ? this._id : null;
    },
    set mId(aVal) {
      delete this._id;
      this._id = aVal;
    },
    
    // parent category id
    get mParentCategoryId() {
      return (typeof(this._parent_category_id) != "undefined") ? this._parent_category_id : null;
    },
    set mParentCategoryId(aVal) {
      delete this._parent_category_id;
      this._parent_category_id = aVal;
    },
    
    // name
    get mName() {
      return (typeof(this._name) != "undefined") ? this._name : null;
    },
    set mName(aVal) {
      delete this._name;
      this._name = aVal;
    },
    
    // description
    get mDescription() {
      return (typeof(this._description) != "undefined") ? this._description : null;
    },
    set mDescription(aVal) {
      delete this._description;
      this._description = aVal;
    },
    
    // color
    get mColor() {
      return (typeof(this._color) != "undefined") ? this._color : null;
    },
    set mColor(aVal) {
      delete this._color;
      this._color = aVal;
    },
    
    // TODO Implement `getParentCategory` method!
    
    
    /**
     * Remove category permanently (from the storage, object remains 
     * how long you will hold it (only id is deleted so mId is null).
     * 
     * @return Boolean
     */
    remove: function() {
      if(this.mId != null) {
        if(!extensions.tbnotes.db.removeCategory(this.mId)) {
          return false;
        }
      }
      
      delete this._id;
      return true;
    },
    
    /**
     * Save/update category's record in the storage. If category hasn't been  
     * saved yet (so id is null) will be category added into the storage and 
     * id updated.
     * 
     * @return Boolean
     */
    save: function() {
      if(this.mId == null) {
        let res = extensions.tbnotes.db.addCategory(this);
        
        if(res != null) {
          return false;
        } else {
          this._id = res;
          return true;
        }
      } else {
        return res = extensions.tbnotes.db.updateCategory(this);
      }
    },
    
    /**
     * Prompts the category as string to the console.
     * @param aTitle {string}
     * @return void
     */
    toConsole: function() {
      CU.reportError("Category: { id: \"" + this.mId + "\", " +
                     "parent_id: \"" + this.mParentCategoryId + "\", " +
                     "name: \"" + this.mName + "\", " + 
                     "description: \"" + this.mDescription + "\", " +
                     "color: \"" + this.mColor + "\" }");
    }
    
  }; // End of CategoryPrototype
  
}).apply(extensions.tbnotes.data);
