/* Copyright (c) 2009 Ondrej Donek.
   See the file LICENSE.txt for licensing information. */

(function() {
  
  /**
   * Prototype object for single note's link.
   * 
   * @param aArgs {mixed}
   * @type object
   */
  this.LinkPrototype = function(aArgs)
  {
    if((aArgs instanceof CI.mozIStorageStatementRow)) {
      this._id = aArgs.id;
      this._note_id = aArgs.note_id;
      this._url = aArgs.url;
      this._title = aArgs.title;
    }
  };
  this.LinkPrototype.prototype = 
  {
    // id
    get mId() {
      return (typeof(this._id) != "undefined") ? this._id : null;
    },
    set mId(aVal) {
      delete this._id;
      this._id = aVal;
    },
    
    // note id
    get mNoteId() {
      return (typeof(this._note_id) != "undefined") ? this._note_id : null;
    },
    set mNoteId(aVal) {
      delete this._note_id;
      this._note_id = aVal;
    },
    
    // title
    get mTitle() {
      return (typeof(this._title) != "undefined") ? this._title : null;
    },
    set mTitle(aVal) {
      delete this._title;
      this._title = aVal;
    },
    
    // url
    get mUrl() {
      return (typeof(this._url) != "undefined") ? this._url : null;
    },
    set mUrl(aVal) {
      delete this._url;
      this._url = aVal;
    },
    
    
    // TODO Implement `getNote` method!
    
    
    
    /**
     * Remove link permanently (from the storage, object remains 
     * how long you will hold it (only id is deleted so mId is null).
     * 
     * @return Boolean
     */
    remove: function() {
      if(this.mId != null) {
        if(!extensions.tbnotes.db.removeLink(this.mId)) {
          return false;
        }
      }
      
      delete this._id;
      return true;
    },
    
    /**
     * Save/update link's record in the storage. If link hasn't been saved yet 
     * (so id is null) will be link added into the storage and id updated.
     * 
     * @return Boolean
     */
    save: function() {
      if(this.mId == null) {
        let res = extensions.tbnotes.db.addLink(this);
        
        if(res != null) {
          return false;
        } else {
          this._id = res;
          return true;
        }
      } else {
        return res = extensions.tbnotes.db.updateLink(this);
      }
    },
    
    /**
     * Prompts the link as string to the console.
     * 
     * @param aTitle {string}
     * @return void
     */
    toConsole: function() {
      CU.reportError("Category: { id: \"" + this.mId + "\", " + 
                     "note_id: \"" + this.mNoteId + "\", " + 
                     "url: \"" + this.mUrl + "\", " +
                     "title: \"" + this.mTitle + "\" }");
    }
    
  }; // End of LinkPrototype
  
}).apply(extensions.tbnotes.data);
