/* Copyright (c) 2009 Ondrej Donek.
   See the file LICENSE.txt for licensing information. */

// Namespace
if(typeof(extensions.tbnotes.data) == "undefined") extensions.tbnotes.data = {};

(function() {
  
  /**
   * Prototype object for single note.
   *
   * @param aArgs {mixed}
   * @type object
   */
  this.NotePrototype = function(aArgs)
  {
    if((aArgs instanceof CI.mozIStorageStatementRow)) {
      this._id = aArgs.id;
      this._target_type = aArgs.target_type;
      this._target_key = aArgs.target_key;
      this._target_name = aArgs.target_name;
      this._category_id = aArgs.category_id;
      this._created = aArgs.created;
      this._updated = aArgs.updated;
      this._title = aArgs.title;
      this._body = aArgs.body;
      this._deleted = ((aArgs.deleted == 1) ? true : false);
    }
  };
  this.NotePrototype.prototype = 
  {
    // id
    get mId() {
      return (typeof(this._id) != "undefined") ? this._id : null;
    },
    set mId(aVal) {
      delete this._id;
      this._id = aVal;
    },
    
    // target type
    get mTargetType() {
      return (typeof(this._target_type) != "undefined") ? this._target_type : null;
    },
    set mTargetType(aVal) {
      delete this._target_type;
      this._target_type = aVal;
    },
    
    // target key
    get mTargetKey() {
      return (typeof(this._target_key) != "undefined") ? this._target_key : null;
    },
    set mTargetKey(aVal) {
      delete this._target_key;
      this._target_key = aVal;
    },
    
    // target name
    get mTargetName() {
      return (typeof(this._target_name) != "undefined") ? this._target_name : null;
    },
    set mTargetName(aVal) {
      delete this._target_name;
      this._target_name = aVal;
    },
    
    // category id
    get mCategoryId() {
      return (typeof(this._category_id) != "undefined") ? this._category_id : null;
    },
    set mCategoryId(aVal) {
      delete this._category_id;
      this._category_id = aVal;
    },
    
    // created
    get mCreated() {
      return (typeof(this._created) != "undefined") ? this._created : null;
    },
    set mCreated(aVal) {
      delete this._created;
      this._created = aVal;
    },
    
    // updated
    get mUpdated() {
      return (typeof(this._updated) != "undefined") ? this._updated : null;
    },
    set mUpdated(aVal) {
      delete this._updated;
      this._updated = aVal;
    },
    
    // title
    get mTitle() {
      return (typeof(this._title) != "undefined") ? this._title : null;
    },
    set mTitle(aVal) {
      delete this._title;
      this._title = aVal;
    },
    
    // body
    get mBody() {
      return (typeof(this._body) != "undefined") ? this._body : null;
    },
    set mBody(aVal) {
      delete this._body;
      this._body = aVal;
    },
    
    /**
     * Returns if note is marked as deleted or not.
     * 
     * @return Boolean
     */
    isMarkedAsDeleted: function() {
      return (typeof(this._deleted) != "undefined") ? this._deleted : false;
    },
    
    /**
     * Mark the note as deleted. Note's database record will be also updated.
     * 
     * @return void
     */
    markDelete: function() {
      if(!this.isMarkedAsDeleted()) {
        this._deleted = true;
        
        if(this.mId != null) {
          extensions.tbnotes.db.deleteNote(this.mId);
        }
      }
    },
    
    /**
     * Ensure that the note is not marked as deleted. Note's database record will 
     * be also updated.
     * 
     * @return void
     */
    unmarkDelete: function() {
      if(this.isMarkedAsDeleted()) {
        this._deleted = false;
        
        if(this.mId != null) {
          extensions.tbnotes.db.undeleteNote(this.mId);
        }
      }
    },
    
    /**
     * Remove note permanently (from the storage, object remains 
     * how long you will hold it (only id is deleted so mId is null).
     * 
     * @return Boolean
     */
    remove: function() {
      if(this.mId != null) {
        if(!extensions.tbnotes.db.removeNote(this.mId)) {
          return false;
        }
      }
      
      delete this._id;
      return true;
    },
    
    /**
     * Save/update note's record in the storage. If note hasn't been saved yet 
     * (so id is null) will be note added into the storage and id updated.
     * 
     * @return Boolean
     */
    save: function() {
      if(this.mId == null) {
        let res = extensions.tbnotes.db.addNote(this);
        
        if(res != null) {
          return false;
        } else {
          this._id = res;
          return true;
        }
      } else {
        return res = extensions.tbnotes.db.updateNote(this);
      }
    },
    
    /**
     * Prompts the note as string to the console.
     * 
     * @param aTitle {string}
     * @return void
     */
    toConsole: function() {
      CU.reportError("Note: { id: \"" + this.mId + "\", " +
                     "target_type: \"" + this.mTargetType + "\", " +
                     "target_key: \"" + this.mTargetType + "\", " +
                     "target_name: \"" + this.mTargetType + "\", " +
                     "category_id: \"" + this.mCategoryId + "\", " +
                     "created: \"" + this.mCreated + "\", " + 
                     "updated: \"" + this.mUpdated + "\", " + 
                     "title: \"" + this.mTitle + "\", " + 
                     "body: \"" + this.mBody + "\", " +
                     "deleted: \"" + this.mDeleted + "\" }");
    }
    
  }; // End of NotePrototype
  
}).apply(extensions.tbnotes.data);
