/* Copyright (c) 2009 Ondrej Donek.
   See the file LICENSE.txt for licensing information. */

/**
 * Main controller object of note dialog
 * 
 * @type Object
 */
var tbNoteDialog = {
	
	/** 
	 * Holds instance of object describing target email message
	 * @type Object|NULL
	 */
	_targetMsg: null,
	
	/**
	 * Holds instance of object representing edited note.
	 * @type Object|NULL
	 */
	_editedNote: null,
	
	/**
	 * Fired when dialog is loading.
	 * 
	 * @return void
	 */
	onLoad: function() 
  {
		let op = "add";
		
		if(typeof(window.arguments[0].inn.note) != "undefined") {
			if(window.arguments[0].inn.note != null) {
				op = "edit";
			}
		}
		
		if(op == "add") {
			// Check if we targeting note to some message
			// TODO When we editing note target message can be blank so load 
			//      subject by self here!
			if(typeof(window.arguments[0].inn.msg) != "undefined") {
				if(window.arguments[0].inn.msg != null) {
					this._targetMsg = window.arguments[0].inn.msg;
				}
			}
		}
		else {
			this._editedNote = window.arguments[0].inn.note;
			//this._editedNote.toConsole();
		}
		
		this._updateUi();
	},
	
	/** 
	 * Updates dialog UI.
	 * 
	 * @return void
	 */
	_updateUi: function() 
  {
		var dlg = document.getElementById("tbnotes-note-dialog");
		var cont = document.getElementById("tbnotedialog_createdCont-hbox");
		var target = document.getElementById("tbnotedialog_target-textbox");
		var created = document.getElementById("tbnotedialog_created-textbox");
		var title = document.getElementById("tbnotedialog_title-textbox");
		var body = document.getElementById("tbnotedialog_body-textbox");
		var links = document.getElementById("tbnotedialog_links-hbox");
		var linksArr = new Array();
		
		if(this._editedNote != null) {
			// We are editing given note
			target.value = this._editedNote.mTargetName;
			target.setAttribute("tooltiptext", this._editedNote.mTargetName);
			created.value = this._editedNote.mCreated;
			title.value = this._editedNote.mTitle;
			body.value = this._editedNote.mBody;
			
			if(cont.hasAttribute("collapsed")) {
				cont.removeAttribute("collapsed");
			}
			
			// TODO Use stringbundles!
			var dlgTitle = dlg.getAttribute("titleEdit") + " #" + this._editedNote.mId;
			dlg.setAttribute("title", dlgTitle);
			
			// TODO Get note links!
		} else {
			// We are adding new note
			
			// Has note any target?
			if(this._targetMsg != null) {
				target.value = this._targetMsg.subject;
				target.setAttribute("tooltiptext", this._targetMsg.subject);
			} else {
				// TODO Use stringbundles!
				target.value = "No target";
			}
			
			// Hide grid row with created textbox
			cont.setAttribute("collapsed", "true");
			dlg.setAttribute("title", dlg.getAttribute("titleAdd"));
		}
		
		if(linksArr.length == 0) {
			// Append 'No links attached' label
			let label = document.createElementNS(XUL_NS, "label");
			// TODO Use stringbundles!
			label.setAttribute("value", "No links attached yet");
			label.setAttribute("class", "noLinksAttached");
			links.appendChild(label);
		}
		else {
			// TODO Display all links
		}
		
		// Append 'Add link' label
		let label = document.createElementNS(XUL_NS, "label");
		// TODO Use stringbundles!
		label.setAttribute("value", "Add link");
		label.setAttribute("class", "link addLinkCmd");
		label.setAttribute("onclick", "tbNoteDialog.doCommand('add_link');");
		links.appendChild(label);
		
		// Select title textbox
		title.focus();
	},
	
	/** 
	 * Fired when user accepted the note dialog.
	 * 
	 * @return boolean
	 */
	onAccept: function() {
		try {
			var titleElm = document.getElementById("tbnotedialog_title-textbox");
			var bodyElm = document.getElementById("tbnotedialog_body-textbox");
			
			// TODO Check if title is empty!
			// TODO Check if body is empty!
			// TODO What if target was cleared by user?
			
			if(this._editedNote == null) {
				let noteObj = null;
				let winMan = CC["@mozilla.org/appshell/window-mediator;1"].getService();
				let winManInt = winMan.QueryInterface(CI.nsIWindowMediator);
				let win = winManInt.getMostRecentWindow("mail:3pane");
				
				with(win) {
					noteObj = new extensions.tbnotes.data.NotePrototype();
				}
				
				// We are adding new note
				this._editedNote = noteObj;
				this._editedNote.mTitle = titleElm.value;
				this._editedNote.mBody = bodyElm.value;
				
				if(this._targetMsg != null) {
					let isNews = this._targetMsg.selectedMessageIsNews;
					let isFeed = this._targetMsg.selectedMessageIsFeed;
					
					if(isNews) {
						this._editedNote.mTargetType = "newsitem";
					} else if(isFeed) {
						this._editedNote.mTargetType = "feeditem";
					} else {
						this._editedNote.mTargetType = "mailmessage";
					}
					// TODO `address book items` => `abitem`
					
					this._editedNote.mTargetKey = this._targetMsg.key;
					this._editedNote.mTargetName = this._targetMsg.subject;
				}
			}
			else {
				// We are editing note
				this._editedNote.mTitle = titleElm.value;
				this._editedNote.mBody = bodyElm.value;
			}
			
			//this._editedNote.toConsole();
			
			window.arguments[0].out = this._editedNote;
		} catch(e) { CU.reportError(e); }
		
		return true;
	}, 
	
	/**
	 * Fired when user canceled the note dialog.
	 * 
	 * @return boolean
	 */
	onCancel: function() {
		var pref = TBNOTES_PREFS_KEY + "dont_show_close_confirm";
		
		// Check if user want to see confirmation dialog
		if(TBNOTES_PREFS.prefHasUserValue(pref) === true) {
			if(TBNOTES_PREFS.getBoolPref(pref) === true) {
				return true;
			}
		}
		
		// Show confirmation if user really want to close unsaved note
		var prompts = CC["@mozilla.org/embedcomp/prompt-service;1"].
				getService(CI.nsIPromptService);
		
		// TODO Use stringbundles!
		var title = "Note dialog";
		var desc = "Do you really want to close unsaved note?";
		var question = "Do not ask me again.";
		var check = { value: false };
		var res = prompts.confirmCheck(window, title, desc, question, check);
		
		TBNOTES_PREFS.setBoolPref(pref, check.value);
		
		return res;
	},
	
	/**
	 * Performs requested command
	 * 
	 * @param aCmdName {string}
	 * @return void
	 */
	doCommand: function(aCmdName) {
		switch(aCmdName) {
			// Add new link command
			case "add_link":
				// Open dialog self
				// TODO Add `params.inn.noteId`!!!
				var params = { inn: {}, out: { title: "", url: "" } };
				window.openDialog(TBNOTES_LINKDLG_URI, "", TBNOTES_LINKDLG_FTRS, params).focus();
				
				if(params.out) {
					CU.reportError("TODO Add link ({ title: '" + params.out.title + "', url: '" + params.out.url + "'})");
					//extensions.tbnotes.db.addNote(params.out);
				}
				break;
			
			default:
				CU.reportError("tbNoteDialog.doCommand:: undefined command '" + aCmdName + "'.");
				break;
		}
	}
	
}; // End of tbNoteDialog
